//+------------------------------------------------------------------+
//|                                                          KDJ.mq4 |
//|                                     Copyright? 2009, Walter Choy |
//|                                                                  |
//+------------------------------------------------------------------+
#property copyright "Copyright? 2009, Walter Choy"
#property link      ""

#property indicator_separate_window
#property indicator_buffers 2
#property indicator_color1  LimeGreen
#property indicator_color2  Red
#property indicator_width1  3
#property indicator_width2  3
#property indicator_minimum 0
#property indicator_maximum 1

//---- input parameters
extern string TimeFrame = "Current time frame";
extern int    nPeriod   = 9;
extern double factor_1  = 0.6666666;
extern double factor_2  = 0.3333333;
extern bool   alertsOn         = true;
extern bool   alertsOnCurrent  = true;
extern bool   alertsMessage    = true;
extern bool   alertsSound      = false;
extern bool   alertsNotify     = true;
extern bool   alertsEmail      = true;
extern string soundFile        = "alert2.wav"; 
//---- buffers
double percentK[];
double percentD[];
double percentJ[];
double RSV[];
double UpH[];
double DnH[];
double trend[];

string indicatorFileName;
bool   returnBars;
int    timeFrame;

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
  {
   IndicatorBuffers(7);
   SetIndexBuffer(0,UpH); SetIndexStyle(0,DRAW_HISTOGRAM);
   SetIndexBuffer(1,DnH); SetIndexStyle(1,DRAW_HISTOGRAM);   
   SetIndexBuffer(2,percentK);
   SetIndexBuffer(3,percentD);
   SetIndexBuffer(4,percentJ);
   SetIndexBuffer(5,RSV);
   SetIndexBuffer(6,trend);
   indicatorFileName = WindowExpertName();
   returnBars        = TimeFrame == "returnBars"; if (returnBars) return(0);
   timeFrame         = stringToTimeFrame(TimeFrame);
   IndicatorShortName(timeFrameToString(timeFrame)+"  KDJ Histo");
   return(0);
  }
//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit()
  {
//----
   
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start()
  {
   int counted_bars=IndicatorCounted();
      if(counted_bars < 0) return(-1);
      if(counted_bars>0) counted_bars--;
         int limit = MathMin(Bars-counted_bars,Bars-1);
         if (returnBars) { percentK[0] = limit+1; return(0); }
         if (timeFrame != Period())
         {
            limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
            for (int i=limit; i>=0; i--)
            {
               int y = iBarShift(NULL,timeFrame,Time[i]);               
                  UpH[i] = iCustom(NULL,timeFrame,indicatorFileName,"",nPeriod,factor_1,factor_2,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsNotify,alertsEmail,soundFile,0,y);
                  DnH[i] = iCustom(NULL,timeFrame,indicatorFileName,"",nPeriod,factor_1,factor_2,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsNotify,alertsEmail,soundFile,1,y);
            }
            return(0);                
         }

   //
   //
   //
   //
   //
      
   int num = Bars - nPeriod;
   for(i = limit; i >= 0; i--)
   {
         double Cn = iClose(NULL,0,i); 
         double Ln = iClose(NULL,0,i); 
         double Hn = iClose(NULL,0,i);
         for(int k=0; k<nPeriod; k++)
         {
            if (Ln > iLow(NULL,0,i+k))  Ln =  iLow(NULL,0,i+k);
            if (Hn < iHigh(NULL,0,i+k)) Hn = iHigh(NULL,0,i+k);
         }

         if (Hn-Ln != 0) RSV[i] = (Cn-Ln)/(Hn-Ln)*100; else RSV[i] = 50;

         if (i >= num) {
            percentK[i] = factor_1 * 50 + factor_2 * RSV[i];
            percentD[i] = factor_1 * 50 + factor_2 * percentK[i];
         } else {
            percentK[i] = factor_1 * percentK[i+1] + factor_2 * RSV[i];
            percentD[i] = factor_1 * percentD[i+1] + factor_2 * percentK[i];
         }
         percentJ[i] = 3 * percentD[i] - 2 * percentK[i];
         
         //
         //
         //
         //
         //
         
         UpH[i] = EMPTY_VALUE;
         DnH[i] = EMPTY_VALUE;
         trend[i] = trend[i+1];
            if (percentK[i]>percentD[i] && percentD[i]>percentJ[i]) trend[i] = 1; 
            if (percentK[i]<percentD[i] && percentD[i]<percentJ[i]) trend[i] =-1; 
            if (trend[i] == 1) UpH[i] = 1;
            if (trend[i] ==-1) DnH[i] = 1;
            
            }
            
            //
            //
            //
            //
            //
      
            if (alertsOn)
            {
              if (alertsOnCurrent)
                   int whichBar = 0;
              else     whichBar = 1; whichBar = iBarShift(NULL,0,iTime(NULL,timeFrame,whichBar));
              if (trend[whichBar] != trend[whichBar+1])
              if (trend[whichBar] == 1)
                    doAlert("Buy");
              else  doAlert("Sell");       
   }
return(0);
}
  
//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = stringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}

//
//
//
//
//

string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string stringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int tchar = StringGetChar(s, length);
         if((tchar > 96 && tchar < 123) || (tchar > 223 && tchar < 256))
                     s = StringSetChar(s, length, tchar - 32);
         else if(tchar > -33 && tchar < 0)
                     s = StringSetChar(s, length, tchar + 224);
   }
   return(s);
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

void doAlert(string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
      if (previousAlert != doWhat || previousTime != Time[0]) {
          previousAlert  = doWhat;
          previousTime   = Time[0];

          //
          //
          //
          //
          //

          message =  StringConcatenate(Symbol()," ",timeFrameToString(timeFrame)," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," KDJ histo ",doWhat);
             if (alertsMessage) Alert(message);
             if (alertsNotify)  SendNotification(StringConcatenate(Symbol(), Period() ," KDJ histo " +" "+message));
             if (alertsEmail)   SendMail(StringConcatenate(Symbol()," KDJ histo "),message);
             if (alertsSound)   PlaySound(soundFile);
      }
}
      